package org.gersteinlab.act.io;

import org.gersteinlab.act.data.*;

import java.io.*;
import java.util.*;

/**
 *	<pre>
 *	This class defines a reader of data files in BED format. See
 *	http://genome.ucsc.edu/FAQ/FAQformat.html for definition.
 *	</pre>
 *
 *	@author		Kevin Yuk-Lap Yip
 *	@version	1.0 (May 7, 2010)
 *
 *	<pre>
 *	Change History:
 *	1.0	- Initial version
 *	</pre>
 */
public class BedFilesReader extends DataFilesReader
{
	/*----------------------------------------------------------------------
	 Public methods
	----------------------------------------------------------------------*/

	/**
	 *	Create a new reader with source data from an input reader.
	 *	@param		rIn			The input reader
	 */
	public BedFilesReader(Reader rIn)
	{
		super(rIn);
	}

	/**
	 *	Read the next interval from the reader.
	 */
	public Interval readInterval() throws IOException
	{
		// Read a line from the input reader
		String line = br.readLine();
		Interval interval = null;

		while (true)
		{
			// Case 1: no more unread lines
			if (line == null)
				break;

			// Case 2: an unused line
			else if (line.length() == 0 || line.startsWith("track"))
			{
				line = br.readLine();
				continue;
			}

			// Case 3: a data line
			else
			{
				// Parse the fields
				StringTokenizer st = new StringTokenizer(line, " \t", false);
				String seg = st.nextToken();	// Treated as chromosome
				long start = Long.parseLong(st.nextToken());
				long end = Long.parseLong(st.nextToken());
				double value = Double.parseDouble(st.nextToken());

				// Create and return the interval
				interval = new Interval(seg, start, end);
				interval.setDoubleValue(value);
				break;
			}
		}
		return interval;
	}
}