package org.gersteinlab.act.io;

import org.gersteinlab.act.data.*;
import org.gersteinlab.act.util.*;

import java.io.*;
import java.util.*;

/**
 *	<pre>
 *	This class defines a reader of data files in narrowPeak format. See
 *	http://www.genome.duke.edu/labs/furey/software/fseq/ for definition.
 *	</pre>
 *
 *	@author		Kevin Yuk-Lap Yip
 *	@version	1.0 (May 7, 2010)
 *
 *	<pre>
 *	Change History:
 *	1.0	- Initial version
 *	</pre>
 */
public class NarrowPeakFilesReader extends DataFilesReader
{
	/*----------------------------------------------------------------------
	 Public methods
	----------------------------------------------------------------------*/

	/**
	 *	Create a new reader with source data from an input reader.
	 *	@param		rIn			The input reader
	 */
	public NarrowPeakFilesReader(Reader rIn)
	{
		super(rIn);
	}

	/**
	 *	Read the next interval from the reader.
	 */
	public Interval readInterval() throws IOException
	{
		// Read a line from the input reader
		String line = br.readLine();
		Interval interval = null;

		while (true)
		{
			// Case 1: no more unread lines
			if (line == null)
				break;

			// Case 2: an unused line
			else if (line.length() == 0 || line.startsWith("track") || line.startsWith("#"))
			{
				line = br.readLine();
				continue;
			}

			// Case 3: a data line
			else
			{
				// Parse the fields
				StringTokenizer2 st = new StringTokenizer2(line, "\t", false);
				String chr = st.nextToken();
				long start = Long.parseLong(st.nextToken());
				long end = Long.parseLong(st.nextToken());
				String name = st.nextToken();
				String score = st.nextToken();
				char strand = st.nextToken().charAt(0);
				double value = Double.parseDouble(st.nextToken());
				String pValue = st.nextToken();
				String qValue = st.nextToken();
				String peak = st.nextToken();

				// Create the interval
				interval = new Interval(chr, start, end, strand);
				interval.setDoubleValue(value);
				interval.setAttribute("name",	name);
				interval.setAttribute("score",	score);
				interval.setAttribute("pValue",	pValue);
				interval.setAttribute("qValue",	qValue);
				interval.setAttribute("peak",	peak);

				// Create and return the interval
				break;
			}
		}
		return interval;
	}
}